@@ -1,32 +1,32 @@
 package Form::Tiny::Filter;
-
+$Form::Tiny::Filter::VERSION = '2.13_1'; # TRIAL
+$Form::Tiny::Filter::VERSION = '2.131';
 use v5.10;
+use strict;
 use warnings;
 use Moo;
 use Types::Standard qw(HasMethods CodeRef);
 
 use namespace::clean;
 
-our $VERSION = '2.00';
-
-has "type" => (
-	is => "ro",
-	isa => HasMethods ["check"],
+has 'type' => (
+	is => 'ro',
+	isa => HasMethods ['check'],
 	required => 1,
 );
 
-has "code" => (
-	is => "ro",
+has 'code' => (
+	is => 'ro',
 	isa => CodeRef,
 	required => 1,
 );
 
 sub filter
 {
-	my ($self, $value) = @_;
+	my ($self, $obj, $value) = @_;
 
 	if ($self->type->check($value)) {
-		return $self->code->($value);
+		return $self->code->($obj, $value);
 	}
 
 	return $value;
@@ -45,7 +45,7 @@ Form::Tiny::Filter - a representation of a filter
 	# in your form class
 
 	# the following will be coerced into Form::Tiny::Filter
-	form_filer Str, sub { uc shift() };
+	form_filter Str, sub { uc pop() };
 
 =head1 DESCRIPTION
 
@@ -61,7 +61,7 @@ Required.
 
 =head2 code
 
-A code reference accepting a single scalar and performing the filtering. The scalar will already be checked against the type.
+A code reference C<($form, $value)> and performing the filtering. The scalar C<$value> will already be checked against the type. Should return modified C<$value>.
 
 Required.
 
@@ -69,6 +69,9 @@ Required.
 
 =head2 filter
 
-Accepts a single scalar, checks if it matches the type and runs the code reference with it as an argument.
+	$filtered = $filter->filter($filtered, $form);
+
+Checks if C<$filtered> matches the type and runs the code reference. C<$form> is a form instance in which the filtering happens, and will be passed before C<$filtered>. The C<$filtered> value is the last parameter to the coderef, so it can be retrieved using C<pop()>.
 
 The return value is the scalar value, either changed or unchanged.
+
