@@ -1,20 +1,21 @@
 package Form::Tiny::Hook;
-
+$Form::Tiny::Hook::VERSION = '2.13_1'; # TRIAL
+$Form::Tiny::Hook::VERSION = '2.131';
 use v5.10;
+use strict;
 use warnings;
 use Moo;
 use Types::Standard qw(Enum CodeRef Bool);
 
 use namespace::clean;
 
-our $VERSION = '1.99_01';
-
 use constant {
 	HOOK_REFORMAT => 'reformat',
 	HOOK_BEFORE_MANGLE => 'before_mangle',
 	HOOK_BEFORE_VALIDATE => 'before_validate',
 	HOOK_AFTER_VALIDATE => 'after_validate',
 	HOOK_CLEANUP => 'cleanup',
+	HOOK_AFTER_ERROR => 'after_error',
 };
 
 my @hooks = (
@@ -22,17 +23,18 @@ my @hooks = (
 	HOOK_BEFORE_MANGLE,
 	HOOK_BEFORE_VALIDATE,
 	HOOK_AFTER_VALIDATE,
-	HOOK_CLEANUP
+	HOOK_CLEANUP,
+	HOOK_AFTER_ERROR,
 );
 
-has "hook" => (
-	is => "ro",
+has 'hook' => (
+	is => 'ro',
 	isa => Enum [@hooks],
 	required => 1,
 );
 
-has "code" => (
-	is => "ro",
+has 'code' => (
+	is => 'ro',
 	isa => CodeRef,
 	required => 1,
 );
@@ -80,7 +82,7 @@ This is a simple class which stores a hook type together with a code reference w
 
 =head2 hook
 
-A hook type. Currently available types are: C<reformat before_mangle before_validate after_validate cleanup>.
+A hook type. Currently available types are: C<reformat before_mangle before_validate after_validate cleanup after_error>.
 
 Required.
 
@@ -99,3 +101,4 @@ A boolean - whether the hook should be inherited to child forms. True by default
 =head2 is_modifying
 
 Given a hook object, this method will return a boolean value which indicates whether the hook is meant to be modifying the passed in value. If false, we should discard the return value of that hook's code reference.
+
