@@ -1,6 +1,8 @@
 package Form::Tiny::Path;
-
+$Form::Tiny::Path::VERSION = '2.13_1'; # TRIAL
+$Form::Tiny::Path::VERSION = '2.131';
 use v5.10;
+use strict;
 use warnings;
 use Moo;
 use Carp qw(croak);
@@ -8,25 +10,22 @@ use Types::Standard qw(ArrayRef);
 
 use namespace::clean;
 
-our $VERSION = '1.99_01';
 
 our $nesting_separator = q{.};
 our $array_marker = q{*};
 our $escape_character = q{\\};
 
-has "path" => (
-	is => "ro",
+has 'path' => (
+	is => 'ro',
 	isa => ArrayRef,
-	writer => "_set_path",
-	coerce => 1,
+	writer => '_set_path',
 	required => 1,
 );
 
-has "meta" => (
-	is => "ro",
+has 'meta' => (
+	is => 'ro',
 	isa => ArrayRef,
-	writer => "_set_meta",
-	lazy => 1,
+	writer => '_set_meta',
 	required => 1,
 );
 
